/**
 * sht31.c
 *
 * Sensirion SHT31-DIS
 * Humidity and Temperature Sensor
 */
#include "project.h"
#include <stdio.h>
#include <string.h>
#include "sht31.h"
#include "i2c_utils.h"

extern char str[] ;

/* Single Shot Commands */
/* With Clock Stretching */
#define SS_CS_H 0x2C06
#define SS_CS_M 0x2C0D
#define SS_CS_L 0x2C10

/* Without Clock Strething */
#define SS_H    0x2400
#define SS_M    0x240B
#define SS_L    0x2416

/* Periodic Data Acquision Commands */
/* 0.5 MPS (Measurement Per Second) */
#define PA_05MPS_H 0x2032
#define PA_05MPS_M 0x2024
#define PA_05MPS_L 0x202F
/* 1 MPS */
#define PA_1MPS_H  0x2130
#define PA_1MPS_M  0x2126
#define PA_1MPS_L  0x212D
/* 2 MPS */
#define PA_2MPS_H  0x2236
#define PA_2MPS_M  0x2220
#define PA_2MPS_L  0x222B
/* 4 MPS */
#define PA_4MPS_H  0x2334
#define PA_4MPS_M  0x2322
#define PA_4MPS_L  0x2329
/* 10 MPS */
#define PA_10MPS_H 0x2737
#define PA_10MPS_M 0x2721
#define PA_10MPS_L 0x272A

/* ART (accelerated Response Time) Command */
#define ART_CMD   0x2B32

/* Break command / Stop Periodic Data Acquisition Mode */
#define BREAK_CMD 0x3090

/* Reset Command */
#define RESET_CMD 0x30A2

/* Heater command */
#define HEATER_ON  0x306D
#define HEATER_OFF 0x3066

/* Read Status Register */
#define READ_STATUS 0xF32D

/* Clear Status Register */
#define CLEAR_STATUS 0x3041

float SHT31_calc_rh(uint16_t srh)
{
    float rh ;
    rh = 100.0 * srh / 65535.0 ;
    return( rh ) ;
}

float SHT31_calc_temp_c(uint16_t sc)
{
    float temp_c ;
    temp_c = -45.0 + 175.0 * (float)sc / 65535.0 ;
    return( temp_c ) ;
}

float SHT31_calc_temp_f(uint16_t sf)
{
    float temp_f ;
    temp_f = -49.0 + 315.0 * (float)sf / 65535.0 ;
    return( temp_f ) ;
}

#define POLYNOMIAL 0x31 
#define CRC_WIDTH 8
uint8_t SHT31_calc_crc(uint16_t data)
{
    uint8_t crc = 0xFF ;
    int i, j ;
    for (j = 0; j < 2 ; j++) { 
        crc ^= ((uint8_t*)(&data))[1-j];
        for (i = CRC_WIDTH ; i ; i--) {
            if (crc & 0x80) {
                crc = (crc << 1) ^ POLYNOMIAL ;
            } else {
                crc <<= 1 ;
            }
        }
    }
    return( crc ) ;
}

int SHT31_WriteCommand(uint16_t command)
{
    int result = 0 ;
    int state = 0 ;
    uint8_t cmd_msb, cmd_lsb ;
    cmd_msb = (command >> 8) & 0xFF ;
    cmd_lsb = command & 0xFF ;
    result = I2C_I2CMasterSendStart(SHT31_I2C_ADDRESS, 0, 1) ;
    if (result) {
        state += 0x01 ;
        sprintf(str, "I2C Send Start: %d\n", result) ;
    }
    result = i2c_writeByte(cmd_msb) ;
    if (result) {
        state += 0x02 ;
        sprintf(str, "I2C Send CMD MSB: %d\n", result) ;
    }    
    result = i2c_writeByte(cmd_lsb) ;
    if (result) {
        state += 0x04 ;
        sprintf(str, "I2C Send CMD LSB: %d\n", result) ;
    }  
    return( state ) ;
}

int SHT31_single_read(uint16_t *temp, uint16_t *hume) 
{
    int result = 0 ;
    uint8_t data[6] ;
    result = SHT31_WriteCommand(SS_CS_H) ;
    CyDelay(10) ;
    
    result = I2C_I2CMasterSendRestart(SHT31_I2C_ADDRESS, 1, 1) ;
    CyDelay(1) ;
    
    data[0] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* temp MSB */
    data[1] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* temp LSB */
    data[2] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* temp CRC */
    data[3] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* hume MSB */
    data[4] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* hume LSB */
    data[5] = i2c_readByte(I2C_I2C_NAK_DATA) ; /* hume CRC */
    I2C_I2CMasterSendStop(1) ;
    
    *temp = (data[0] << 8) | data[1] ;
    sprintf(str, "\tTemp   = 0x%04X, crc = 0x%02X ", *temp, data[2]) ;
    UART_UartPutString(str) ;
    if (data[2] == SHT31_calc_crc(*temp)) {
        UART_UartPutString("OK\n") ;
    } else {
        UART_UartPutString("Error!\n") ;
        result |= 0x01 ;
    }
    *hume = (data[3] << 8) | data[4] ;
    sprintf(str, "\tHume   = 0x%04X, crc = 0x%02X ", *hume, data[5]) ; 
    UART_UartPutString(str) ;
    if (data[5] == SHT31_calc_crc(*hume)) {
        UART_UartPutString("OK\n") ;
    } else {
        UART_UartPutString("Error!\n") ;
        result |= 0x02 ;
    }
    return(result) ;
}

int SHT31_reset(void) 
{
    int result = 0 ;
    result = SHT31_WriteCommand(RESET_CMD) ;
    if (result) {
        sprintf(str, "Reset Status: %d\n", result) ;
        UART_UartPutString(str) ;
    }
    I2C_I2CMasterSendStop(1) ;
    return(result) ;
}

int SHT31_read_status(uint16_t *status) 
{
    int result = 0 ;
    uint8_t data[3] ;
    
    result = SHT31_WriteCommand(READ_STATUS) ;
    CyDelay(10) ;
    
    result = I2C_I2CMasterSendRestart(SHT31_I2C_ADDRESS, 1, 1) ;
    CyDelay(1) ;
    
    data[0] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* status MSB */
    data[1] = i2c_readByte(I2C_I2C_ACK_DATA) ; /* status LSB */
    data[2] = i2c_readByte(I2C_I2C_NAK_DATA) ; /* status CRC */
    I2C_I2CMasterSendStop(1) ;
    
    *status = (data[0] << 8) | data[1] ;
    sprintf(str, "\tStatus = 0x%04X, crc = 0x%02X ", *status, data[2]) ;
    UART_UartPutString(str) ;
    if (data[2] == SHT31_calc_crc(*status)) {
        UART_UartPutString("OK\n") ;
    } else {
        UART_UartPutString("Error!\n") ;
        result++ ;
    }
    return(result) ;
}

int SHT31_clear_status(void) 
{
    int result = 0 ;

    result = SHT31_WriteCommand(CLEAR_STATUS) ;
    I2C_I2CMasterSendStop(1) ;
    return(result) ;
}

int SHT31_heater(int enable) 
{
    int result ;
    if (enable) {
        result = SHT31_WriteCommand(HEATER_ON) ;
    } else {
        result = SHT31_WriteCommand(HEATER_OFF) ;
    }
    return( result ) ;
}
